==> Mudlib-CD.00.28.tar.gz: lib/std/room/exits.c <==
/*

  /std/room/exits.c


   ----------------------------------------------------------------------

   Defined functions and variables:

   add_exit(string,string,delay,tired)     Adds an additional exit to the room.
                                     add_exit can be called repeatedly to
				     add more exits. First string is the
				     destination room, second is the command
				     given to move and the third (negative) is
				     the walking delay value, if it is > 0
				     then the exit can not be used. This is
				     usually used as a 'value by function call'
				     to a function in the room. tired is the
				     value to subtract from the players, when 
				     he walks that way, fatigue.

   remove_exit(string)               Removes one exit from the list of exits.
                                     string is the commandname of the exit.
				     Returns 1 if successfully removed.

   query_exit()		             Gives the defined exits.

   query_tired_exits()		     Returns the tired array.


   ----------------------------------------------------------------------

*/

static string   *room_exits;	/* "room/church", "north" , "exitfun" ... */
static string   *tired_exits;   /* Value, how tired the player should become */
static	int	load_exits = 0;
/*
 * Fix to get rid of the obnoxius 'What ?' when we try to walk in a nonexistant
 * direction. (Should northeast, and the rest be included ?) Yes.
 */
#define DEF_DIRS ({ "north", "south", "east", "west", "up", "down", \
		    "northeast", "northwest", "southeast", "southwest" })

/* Function name: ugly_update_function
 * Description:	  Just fix so the verb is updated in a room.
 * Arguments:	  cmd - the command verb
 *		  add - add command if 1 else remove.
 */
public void
ugly_update_action(string cmd, int add)
{
    object *ob, old_tp;
    int i;

    old_tp = this_player();
    ob = all_inventory(this_object());
    for (i == 0; i < sizeof(ob); i++)
    {
	if (living(ob[i]))
	{
	    set_this_player(ob[i]);
    	    if (add == 1)
	    	add_action("unq_move", cmd);
    	    else
	    	add_action("unq_no_move", cmd);
	}
    }
    if(old_tp)
	set_this_player(old_tp);
}

/*
 * Function name:
 * Description:     Add direction commands to livings in the room
 */
public void
init()
{
    int i;
    string *dd;

    ::init();

    for (i = 1; i < sizeof(room_exits); i += 3)
	add_action("unq_move", room_exits[i]);

    dd = DEF_DIRS;
    for (i = 0; i < sizeof(dd); i++)
	if (member_array(dd[i], room_exits) < 0)
	    add_action("unq_no_move", dd[i]);

#ifdef Genesis_jnatest
    /*
       If we load the neighbour rooms here they are ready when entered
    */
    if (!load_exits)
    {
	for (i = 0; i < sizeof(room_exits); i += 3)
	{
	    if (file_size(room_exits[i]) > 0)
		catch(room_exits[i]->teleledningsanka());
	}
	load_exits = 1;
    }
#endif
}


/*
 * Function name: add_exit
 * Description:   Add one exit to the room. (see also unq_move)
 * Arguments:	  place: The filename of the room to move to
 *                cmd:   The command given to move
 *                efunc: The delay value often given by 'value by function
 *			 call'. The delayvalues are:
 *			 0: Direct move to destination
 *			 =1: Can not move in this dir, do not try others
 *			 >1: Can not move to destination, but try the rest
 *			 <0: Move is done with delay (see link_room)
 *		  tired: How much more tired the player should become from
 *			 walking in that direction. Default is 1.
 */
public varargs int
add_exit(string place, string cmd, mixed efunc, mixed tired)
{
    if (query_prop(ROOM_I_NO_EXTRA_EXIT))
	return 0;
    if (pointerp(room_exits))
	room_exits = room_exits + ({ place, cmd, efunc });
    else
	room_exits = ({ place, cmd, efunc });
    if (!pointerp(tired_exits))
	tired_exits = ({});
    if (tired)
	tired_exits += ({ tired });
    else
	tired_exits += ({ 1 }); /* Standard cost for walking.*/

/*    update_actions(); */
    ugly_update_action(cmd, 1);
    return 1;
}


/*
 * Function name: remove_exit
 * Description:   Remove one exit from the room.
 * Arguments:	  cmd:   The command given to move in the exit to be removed
*/
public int
remove_exit(string cmd)
{
    int             i;

    if (!pointerp(room_exits))
	return 0;
    if (query_prop(ROOM_I_NO_EXTRA_EXIT))
	return 0;

    for (i = 1; i < sizeof(room_exits); i += 3)
	if (cmd == room_exits[i])
	{
	    room_exits = exclude_array(room_exits, i - 1, i + 1);
/*	    update_actions(); */
	    ugly_update_action(cmd, 0);
	    return 1;
	}
    return 0;
}


/*
 * Function name: query_exit
 * Description:   Gives a list of the possible exits from this room.
 * Returns:       An array on the form below (example):
 *
 * ({
 *    "/room/church", "north" , "@@exitfun" ,
 *    "/room/post", "post" , "@@exitfun" ,
 * })
 *
 */
public mixed       /* exitfun may be an int */
query_exit()
{
    if (room_exits)
	return slice_array(room_exits, 0, sizeof(room_exits));
    else
	return 0;
}

/*
 * Function name: query_tired_exits
 * Description:   Gives a list of the tired values for each exit.
 * Returns:       The array, one element per exit.
 */
public mixed
query_tired_exits() { return tired_exits + ({}); }

/*
 * Function name: query_exit_rooms
 * Description:   Returns an array of strings containing the full path names
 *                to all rooms that are connected to this one by add_exit()'s.
 * Returns:       A string array
 */
string *
query_exit_rooms()
{
    mixed *exit_values;
    int exit_arrays_counter;
    string *exit_rooms;

    exit_rooms = ({});
    exit_values = this_object()->query_exit();
    if (sizeof(exit_values))
    {
        for (exit_arrays_counter = 0;
             exit_arrays_counter < (sizeof(exit_values)) - 1;
             exit_arrays_counter += 3)
        {
            exit_rooms += ({ exit_values[exit_arrays_counter] });
        }
    }
    return exit_rooms;
}

/*
 * Function name: query_exit_cmds
 * Description:   Returns an array of strings that the player can use as
 *                commands to move through an exit added by add_exit().
 * Returns:       A string array
 */
string *
query_exit_cmds()
{
    mixed *exit_values;
    int exit_arrays_counter;
    string *exit_cmds;

    exit_cmds = ({});
    exit_values = this_object()->query_exit();
    if (sizeof(exit_values))
    {
        for (exit_arrays_counter = 0;
             exit_arrays_counter < (sizeof(exit_values)) - 1;
             exit_arrays_counter += 3)
        {
            exit_cmds += ({ exit_values[1 + exit_arrays_counter] });
        }
    }
    return exit_cmds;
}

/*
 * Function name: query_exit_functions
 * Description:   Returns an array of the functions called when a player goes
 *                through an exit added by add_exit().
 * Returns:       A mixed array (strings or integers (0 or 1))
 */
mixed *
query_exit_functions()
{
    mixed *exit_values, *exit_functions;
    int exit_arrays_counter;

    exit_functions = ({});
    exit_values = this_object()->query_exit();
    if (sizeof(exit_values))
    {
        for (exit_arrays_counter = 0;
             exit_arrays_counter < (sizeof(exit_values)) - 1;
             exit_arrays_counter += 3)
        {
            exit_functions += ({ exit_values[2 + exit_arrays_counter] });
        }
    }
    return exit_functions;
}

==> Mudlib-CD.00.30.tar.gz: lib/std/room/exits.c <==
/*

  /std/room/exits.c


   ----------------------------------------------------------------------

   Defined functions and variables:

   add_exit(string,string,delay,tired)     Adds an additional exit to the room.
                                     add_exit can be called repeatedly to
				     add more exits. First string is the
				     destination room, second is the command
				     given to move and the third (negative) is
				     the walking delay value, if it is > 0
				     then the exit can not be used. This is
				     usually used as a 'value by function call'
				     to a function in the room. tired is the
				     value to subtract from the players, when 
				     he walks that way, fatigue.

   remove_exit(string)               Removes one exit from the list of exits.
                                     string is the commandname of the exit.
				     Returns 1 if successfully removed.

   query_exit()		             Gives the defined exits.

   query_tired_exits()		     Returns the tired array.


   ----------------------------------------------------------------------

*/

static string   *room_exits;	/* "room/church", "north" , "exitfun" ... */
static string   *tired_exits;   /* Value, how tired the player should become */
static	int	load_exits = 0;
/*
 * Fix to get rid of the obnoxius 'What ?' when we try to walk in a nonexistant
 * direction. (Should northeast, and the rest be included ?) Yes.
 */
#define DEF_DIRS ({ "north", "south", "east", "west", "up", "down", \
		    "northeast", "northwest", "southeast", "southwest" })

/*
 * Function name: ugly_update_function
 * Description:	  Just fix so the verb is updated in a room.
 * Arguments:	  cmd - the command verb
 *		  add - add command if 1 else remove.
 */
public void
ugly_update_action(string cmd, int add)
{
    object *ob, old_tp;
    int i;

    old_tp = this_player();
    ob = all_inventory(this_object());
    for (i = 0; i < sizeof(ob); i++)
    {
	if (living(ob[i]))
	{
	    set_this_player(ob[i]);
    	    if (add == 1)
	    	add_action("unq_move", cmd);
    	    else
	    	add_action("unq_no_move", cmd);
	}
    }
    if(old_tp)
	set_this_player(old_tp);
}

/*
 * Function name:
 * Description:     Add direction commands to livings in the room
 */
public void
init()
{
    int i;
    string *dd;

    ::init();

    for (i = 1; i < sizeof(room_exits); i += 3)
	add_action("unq_move", room_exits[i]);

    dd = DEF_DIRS;
    for (i = 0; i < sizeof(dd); i++)
	if (member_array(dd[i], room_exits) < 0)
	    add_action("unq_no_move", dd[i]);

#ifdef Genesis_jnatest
    /*
       If we load the neighbour rooms here they are ready when entered
    */
    if (!load_exits)
    {
	for (i = 0; i < sizeof(room_exits); i += 3)
	{
	    if (file_size(room_exits[i]) > 0)
		catch(room_exits[i]->teleledningsanka());
	}
	load_exits = 1;
    }
#endif
}


/*
 * Function name: add_exit
 * Description:   Add one exit to the room. (see also unq_move)
 * Arguments:	  place: The filename of the room to move to
 *                cmd:   The command given to move
 *                efunc: The delay value often given by 'value by function
 *			 call'. The delay values are:
 *			 0: Direct move to destination
 *			 =1: Can not move in this dir, do not try others
 *			 >1: Can not move to destination, but try the rest
 *			 <0: Move is done with delay (see link_room)
 *		  tired: How much more tired the player should become from
 *			 walking in that direction. Default is 1.
 */
public varargs int
add_exit(string place, string cmd, mixed efunc, mixed tired)
{
    if (query_prop(ROOM_I_NO_EXTRA_EXIT))
	return 0;
    if (pointerp(room_exits))
	room_exits = room_exits + ({ place, cmd, efunc });
    else
	room_exits = ({ place, cmd, efunc });
    if (!pointerp(tired_exits))
	tired_exits = ({});
    if (tired)
	tired_exits += ({ tired });
    else
	tired_exits += ({ 1 }); /* Standard cost for walking.*/

/*    update_actions(); */
    ugly_update_action(cmd, 1);
    return 1;
}


/*
 * Function name: remove_exit
 * Description:   Remove one exit from the room.
 * Arguments:	  cmd:   The command given to move in the exit to be removed
 */
public int
remove_exit(string cmd)
{
    int             i;

    if (!pointerp(room_exits))
	return 0;
    if (query_prop(ROOM_I_NO_EXTRA_EXIT))
	return 0;

    for (i = 1; i < sizeof(room_exits); i += 3)
	if (cmd == room_exits[i])
	{
	    room_exits = exclude_array(room_exits, i - 1, i + 1);
/*	    update_actions(); */
	    ugly_update_action(cmd, 0);
	    return 1;
	}
    return 0;
}


/*
 * Function name: query_exit
 * Description:   Gives a list of the possible exits from this room.
 * Returns:       An array on the form below (example):
 *
 * ({
 *    "/room/church", "north" , "@@exitfun" ,
 *    "/room/post", "post" , "@@exitfun" ,
 * })
 *
 */
public mixed       /* exitfun may be an int */
query_exit()
{
    if (room_exits)
	return slice_array(room_exits, 0, sizeof(room_exits));
    else
	return 0;
}

/*
 * Function name: query_tired_exits
 * Description:   Gives a list of the tired values for each exit.
 * Returns:       The array, one element per exit.
 */
public mixed
query_tired_exits() { return tired_exits + ({}); }

/*
 * Function name: query_exit_rooms
 * Description:   Returns an array of strings containing the full path names
 *                to all rooms that are connected to this one by add_exit()'s.
 * Returns:       A string array
 */
string *
query_exit_rooms()
{
    mixed *exit_values;
    int exit_arrays_counter;
    string *exit_rooms;

    exit_rooms = ({});
    exit_values = this_object()->query_exit();
    if (sizeof(exit_values))
    {
        for (exit_arrays_counter = 0;
             exit_arrays_counter < (sizeof(exit_values)) - 1;
             exit_arrays_counter += 3)
        {
            exit_rooms += ({ exit_values[exit_arrays_counter] });
        }
    }
    return exit_rooms;
}

/*
 * Function name: query_exit_cmds
 * Description:   Returns an array of strings that the player can use as
 *                commands to move through an exit added by add_exit().
 * Returns:       A string array
 */
string *
query_exit_cmds()
{
    mixed *exit_values;
    int exit_arrays_counter;
    string *exit_cmds;

    exit_cmds = ({});
    exit_values = this_object()->query_exit();
    if (sizeof(exit_values))
    {
        for (exit_arrays_counter = 0;
             exit_arrays_counter < (sizeof(exit_values)) - 1;
             exit_arrays_counter += 3)
        {
            exit_cmds += ({ exit_values[1 + exit_arrays_counter] });
        }
    }
    return exit_cmds;
}

/*
 * Function name: query_exit_functions
 * Description:   Returns an array of the functions called when a player goes
 *                through an exit added by add_exit().
 * Returns:       A mixed array (strings or integers (0 or 1))
 */
mixed *
query_exit_functions()
{
    mixed *exit_values, *exit_functions;
    int exit_arrays_counter;

    exit_functions = ({});
    exit_values = this_object()->query_exit();
    if (sizeof(exit_values))
    {
        for (exit_arrays_counter = 0;
             exit_arrays_counter < (sizeof(exit_values)) - 1;
             exit_arrays_counter += 3)
        {
            exit_functions += ({ exit_values[2 + exit_arrays_counter] });
        }
    }
    return exit_functions;
}
